#import "WGController.h"
#import "WGMainController.h"

@implementation WGController

- (id)init
{
    self = [super init];

    [NSBundle loadNibNamed:@"WGController" owner:self];

    return self;
}

- (void)dealloc
{
    [filename release];
    [super dealloc];
    return;
}

- (void)awakeFromNib
{

    [tabView setDelegate:self];

    [window setFrameAutosaveName:@"WebGrabber"];
    [window setFrameUsingName:@"WebGrabber"];

    [window makeKeyAndOrderFront:self];
    [window setDelegate:self];
    
    return;
}

- (NSArray *)optionsForRun
{
    NSMutableArray *retArray = [NSMutableArray array];
    // ask my children for their options and build one big array ...
    [retArray addObjectsFromArray:[downloadController optionsForRun]];
    [retArray addObjectsFromArray:[standardController optionsForRun]];
    [retArray addObjectsFromArray:[permissionsController optionsForRun]];
    // pass that array back ...
    return [NSArray arrayWithArray:retArray];
}

- (NSArray *)urlsForRun
{
    NSMutableArray *retArray = [NSMutableArray array];
    // ask my children for their options and build one big array ...
    [retArray addObjectsFromArray:[standardController urlsForRun]];
    // pass that array back ...
    return [NSArray arrayWithArray:retArray];
}

- (void)loadFromDictionary:(NSDictionary *)aDict
{
    [downloadController loadFromDictionary:aDict];
    [standardController loadFromDictionary:aDict];
    [permissionsController loadFromDictionary:aDict];
    return;
}

- (void)saveDocument:(id)sender
{
    // if I have a filename use it, otherwise do a saveAs:
    if (filename && ![filename isEqual:@"Untitled"]) {
        [self saveToFile:filename];
    } else {
        [self saveDocumentAs:self];
    }
    return;
}

- (void)savePanelDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo
{
    if (returnCode == NSOKButton) {
        [self saveToFile:[contextInfo filename]];
    }
    
    if (closing) {
        [window close];
    }
}


- (void)saveDocumentAs:(id)sender
{
    // get a file name and save to that

    NSSavePanel *sPanel = [NSSavePanel savePanel];
    int result;

    [sPanel setAccessoryView:nil];
    [sPanel setRequiredFileType:@"wg"];
    
    [sPanel beginSheetForDirectory:NSHomeDirectory() file:nil modalForWindow:window modalDelegate:self didEndSelector:@selector(savePanelDidEnd:returnCode:contextInfo:) contextInfo:sPanel];

    return;
}

- (void)saveToFile:(NSString *)filePath
{
    NSMutableDictionary *finalDict = [NSMutableDictionary dictionary];
    
    // get each saveDict
    [finalDict addEntriesFromDictionary:[downloadController saveDict]];
    [finalDict addEntriesFromDictionary:[standardController saveDict]];
    [finalDict addEntriesFromDictionary:[permissionsController saveDict]];

    [self setFilename:filePath];
    
    // write that dict to a save file, or to self if need be
    if (![finalDict writeToFile:filePath atomically:NO]) {
        // throw an error
    }

    [window setDocumentEdited:NO];
    return;
}

- (void)setFilename:(NSString *)aFilename
{
    [filename release];
    filename = [aFilename retain];
    [window setTitleWithRepresentedFilename:filename];
    return;
}

- (void)setDirty:(id)sender
{
    [window setDocumentEdited:YES];
}

- (void)setParent:(id)aParent
{
    parent = aParent;
}

- (NSWindow *)window
{
    return window;
}

- (void)closeWindow
{
    [window close];
}

// window delegation
- (BOOL)windowShouldClose:(NSNotification *)aNotification
{
    if ([window isDocumentEdited]) {
        NSBeginAlertSheet(@"Dirty WebGrabber file", @"Save", @"Throw Away", nil, [self window], self, @selector(sheetDidEnd:returnCode:contextInfo:), nil, nil, @"Do you wish to save %@?", filename);
        return NO;
    } else {
        [parent closeDocument:self];
        return YES;
    }
}

- (void)sheetDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo
{
    [NSApp endSheet:sheet];
    if (returnCode) {
    
        [NSTimer scheduledTimerWithTimeInterval:0.0 target:self selector:@selector(saveDocument:) userInfo:nil repeats:NO];
        closing = YES;
    } else {
        [window close];
    }
}


- (id)standardController
{
    return standardController;
}

- (id)permissionsController
{
    return permissionsController;
}

- (id)downloadController
{
    return downloadController;
}

@end
