#import "WGMainController.h"

@implementation WGMainController

- (id)init
{
    openControllers = [NSMutableArray new];
    [self newController:self];
    [NSApp setServicesProvider:self];

    return [super init];
}

- (void)dealloc
{
    [openControllers release];
    [super dealloc];
    return;
}

- (void)newController:(id)sender
{
    [self newControllerWithFilename:nil];
    return;
}

- (void)openDocument:(id)sender
{
    NSArray *fileTypes = [NSArray arrayWithObject:@"wg"];
    NSOpenPanel *oPanel = [NSOpenPanel openPanel];
    int result;

    [oPanel setAccessoryView:nil];
    [oPanel setAllowsMultipleSelection:NO];
    result = [oPanel runModalForDirectory:NSHomeDirectory() file:nil
        types:fileTypes];

    if (result == NSOKButton) {
        [self newControllerWithFilename:[oPanel filename]];
    }
    return;
}

- (void)closeDocument:(id)aDocument
{
    [openControllers removeObject:aDocument];
    [aDocument autorelease];
    return;
}

- (BOOL)application:(NSApplication *)sender openFile:(NSString *)filename
{
    if ([[filename pathExtension] isEqual:@"wg"]) {
        [self newControllerWithFilename:filename];
    	return YES;
    }
    return NO;
}

- (BOOL)applicationShouldTerminate:(NSApplication *)app
{
    NSEnumerator *winEnum = [openControllers objectEnumerator];
    id aController = nil;

    while (aController = [winEnum nextObject]) {
        [aController closeWindow];
    }
    return YES;
}

- (void)newControllerWithFilename:(NSString *)filename
{
    id aNewController = [[WGController alloc] init];
    if (filename) {
        [aNewController loadFromDictionary:[NSDictionary dictionaryWithContentsOfFile:filename]];
        [aNewController setFilename:filename];
    } else {
        [aNewController setFilename:@"Untitled"];
        [aNewController loadFromDictionary:[NSDictionary dictionaryWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"Default" ofType:@"wg"]]];
        [aNewController setDirty:self];
    }
    [aNewController setParent:self];
    [openControllers addObject:aNewController];
    return;
}

- (void)newControllerWithURL:(NSString *)url;
{
    return;
}

- (void)newControllerWithURL:(NSPasteboard *)pboard userData:(NSString *)data error:(NSString **)error
{

    NSString *pboardString;
    NSArray *types;
    id aNewController = [[WGController alloc] init];


    types = [pboard types];


    if (![types containsObject:NSStringPboardType] || !(pboardString = [pboard stringForType:NSStringPboardType])) {
        *error = NSLocalizedString(@"Error: Pasteboard doesn't contain a string.",
                   @"Pasteboard couldn't give string.");
        return;
    }

    [aNewController setFilename:@"Untitled"];
    [aNewController loadFromDictionary:[NSDictionary dictionaryWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"Default" ofType:@"wg"]]];
    [aNewController setDirty:self];
    [aNewController setParent:self];
    [[aNewController standardController] setURL:pboardString];
    [openControllers addObject:aNewController];

    return;
}

- (void)displayAboutWindow:sender
{
    id dict = [NSMutableDictionary dictionary];

    // These are hard coded because there are AppKit/NSApp specific, not Fire

    [dict setObject:@"(c) 1999-2001 epicware, Inc, Eric Peyton" forKey:@"Copyright"];

    [NSApp orderFrontStandardAboutPanelWithOptions:dict];

    return;
}

@end
